%%%%%%---MASTER FANTASTIC---%%%%%%%%

% Updated 7. apr.
% Changelog:
% 1. Converted time in port and slow zones to hours instead of days
% 2. Implemented first attempt of calculating additional Cs-values for all ships
%    Adjusments needed: Cs-values now include wetted surface and factor 2?
%                       -> 2.06 instead of 0.00045
% Updated 8. apr.
% Changelog:
% 1. Jon added for/if loop for calculating stillwaterdragcoefficients
% 2. Completed changes to calculate emission for different vesselspeeds
%
% Updated 9. apr.
% 1. Added formula for calculating varying time with different speeds
% 2. Added subplot to show both cost and emission in plot
%
% Updated 10. apr
% 1. Added K to the Csref-eq.
% 2. Added upper and lower limit for power
% 3. Added 15 percent extra fuel for sea conditions
% 4. TC fixed in frontend
% 5. 
%
% Updated 13. apr
% 1. Removed ij from K-parameter in Csref, to avoid overestimate of Csref
% 2. Updated rest of code to accomodate for 30 knots
% 3. Added functionality where n is reduced for lower speed in such a way
% that it increases the req. power for lower speeds.
%
% Updated 13. apr
% 1. Tt covers a roundtrip, not just a one-way trip
%
% Updated 15. apr
% 1. Divided $/ton with 10^6 to get $/gram so it fits with Fuel (grams)
%
% Updated 20. apr
% 1. Changed 1 + r / 365 to (r / 100) / 365 as the first term include the
% the invested amount as well as interests.
% 2. Changed dimentions of some matrices to match the 60x30 the rest has.
% 3. Cleaned up code; added vmax and comments

%File read - Parameters
Data =  xlsread('FrontendBlock.xlsx', 2, 'D5:AH64');

%Constants
x    =   60;      % Number of ship type entries in Excel document
vmax =   30;      % Number or speed iterations
MCR  =   0.75;    % Maximum continous revolution at design speed
ktms =   1.9438;   % Knots to m/s conversion factor
p    =   1.025;   % Density of saltwater 
g    =   9.81;    % Gravity
v    =   1:vmax;  % Vessel speed, 1 through 30 knots

for i = 1:x 
No(i,1)   =   Data(i,1);          % Number of ships
Voy(i,1)  =   Data(i,2);          % Distance per voyage
Day(i,1)  =   Data(i,3);          % Days per voyage
Tps(i,1)  =   (Data(i,4))*24;     % Time in port and slow zones
vs(i,1)   =   Data(i,5);          % Vessel Speed 
vd(i,1)   =   Data(i,6);          % Design Speed
kW(i,1)   =   Data(i,7);          % Engine size (Installed power)    
n(i,1)    =   Data(i,8);          % Engine efficiency
Paux(i,1) =   Data(i,9);          % Aux. Power
Dis(i,1)  =   Data(i,11);         % Displacement of water (tons)
B(i,1)    =   Data(i,12);         % Width of ships at water line
L(i,1)    =   Data(i,13);         % Length of ship at water line
Dft(i,1)  =   Data(i,14);         % Draft - Fully loaded
Ct(i,1)   =   Data(i,16);         % Drag coefficient
Kf(i,1)   =   Data(i,17);         % Gram fuel per kWh
H13(i,1)  =   Data(i,18);         % Wave height
u(i,1)    =   Data(i,19);         % Wave Speed
Caw(i,1)  =   Data(i,20);         % Wave Drag
Ke(i,1)   =   Data(i,21);         % CO2 per unit fuel
M(i,1)    =   Data(i,22);         % Average weigth of cargo
Chfo(i,1) =   Data(i,23);         % Cost of heavy fuel oil
Cmdo(i,1) =   Data(i,24);         % Cost of marine diesel oil
TC(i,1)   =   Data(i,25);         % Time charter cost per day
CM(i,1)   =   Data(i,26);         % Export value per ton
Cir(i,1)  =   Data(i,27);         % Annual interest
j(i,1)    =   Data(i,28);         % Propeller constant, speed independent
k(i,1)    =   Data(i,29);         % Propeller constant, speed dependent
Cv(i,1)   =   Data(i,30);         % Cargo voyages
Bv(i,1)   =   Data(i,31);         % Ballast voyages
end

% ADDITIONAL DATA
% Distance per roundtrip;                                            
D    =   2 .* Voy;       

% Time per roundtrip
T    =   2 .* Day;     

% Minimum / idle engine load
kWmin = 0.15 .* kW;        

% Time in ports and slow zones
Pps = kWmin;

% Calculation for time spent per voyage per boat, at different speeds
for i = 1:x %1 to X number of ships

    for h = 1:vmax %1 to vmax number of speeds
   
        Tt(i,h) =  ((D(i)/h)/24);
         
    end 
end


%% BLOCK COEFFICIENT SCENARIO MODIFICATIONS

for i = 1:x %1 to X number of ships
    for h = 1:vmax %1 to vmax number of speeds
   
       %Reference Values
% Frouds Number, Initial
Fnref(i,1)   =   ((vd(i,1))/ktms)/(sqrt(g*(L(i,1))));  

% Wetted surface, initial 
Sref(i,1) = ((B(i,1)*L(i,1)) + (2*L(i,1)*Dft(i,1))); 

% Block coefficient, initial
Cbref(i,1) = Dis(i,1) / (L(i,1) * (B(i,1)) * (Dft(i,1)));  

% Scenario 1 - 14% reduction in DWT and Displacement
% Reduced displacement
Dis2(i,1) = (Dis(i,1)) * 0.86;
% Reduced Cargo capacity
M2(i,1) = (M(i,1)) * 0.86;    
% Reduced Time Charter
TC2(i,1) = TC(i,1) * 0.86;   
% Reduced Wetted surface
S2(i,1) = (((B(i,1)*L(i,1)))*0.72) + (2*L(i,1)*Dft(i,1));  

%Scenario 2 - 26% reduction in DWT and Displacement
% Reduced displacement
Dis3(i,1) = (Dis(i,1)) * 0.74; 
% Reduced Cargo capacity
M3(i,1) = (M(i,1)) * 0.74; 
% Reduced Time Charter
TC3(i,1) = TC(i,1) * 0.74;  
% Reduced Wetted surface
S3(i,1) = (((B(i,1)*L(i,1)))*0.52) + (2*L(i,1)*Dft(i,1)); 

%Scenario 3 - Increased Beam 
% Increased width
B1(i,1) = (B(i,1)) * 1.3;                                  

% Corresponding  new block coefficients
Cb1(i,1) = Dis(i,1) / (L(i,1) * (B1(i,1)) * (Dft(i,1)));
Cb2(i,1) = Dis2(i,1) / (L(i,1) * (B(i,1)) * (Dft(i,1)));
Cb3(i,1) = Dis3(i,1) / (L(i,1) * (B(i,1)) * (Dft(i,1)));

    end   
end


%% Section 2 - Propeller efficiencies and still water resistance factors


% 2-1 Change in propeller efficientcy for lower speeds
for ij=1:vmax
    if  v(1,ij) > vd(i,1);
        Ku(i,ij) = n(i,1);
    else
        Ku(i,ij) = n(i,1) * (j(i,1) + (k(i,1) * sqrt(v(1,ij)/vd(i,1))));
    end 
    
        % 2-2 Propeller efficiency
        K(i,ij) = 1 / (Ku(i,ij));   
end

% 2-3 Still Water Resistance Factor
Csref(i,1)=((kW(i,1))*MCR*(1/(K(i,ij)))*2)/(p*(Sref(i,1))*((vd(i,1))^3)); 

% 2-4 Calculations for still water drag coefficients 
for i = 1:x %1 to X number of ships
    for h = 1:vmax %1 to vmax number of speeds
    if h > vd(i); 
        Cs(i,h) = Csref(i) * (0.8 + (0.2*(h/vd(i))^2));     
    else
        if h < vd(i);
            Cs(i,h) = (Csref(i)*0.9) + (((h/vd(i))*0.1)*Csref(i));   
        else
            Cs(i,h) = Csref(i);
        end
    end 
    end   
end


%% Section 3 - Power
%Iterate through 1 to X number of ships
for i = 1:x 
    for ij = 1:vmax %1 to vmax number of speeds
        
% 3-1 Still water power (Ps)
% Reference still water power        
Psref(i,ij) = (p*Cs(i,ij)*Sref(i)*((v(ij))^3))/2;      

% Still water power Scenario 1
Ps2(i,ij) = Psref(i,v(ij))*(((S2(i))*(Cb2(i)))/((Sref(i))*(Cbref(i))));      
% Still water power Scenario 2
Ps3(i,ij) = Psref(i,v(ij))*(((S3(i))*(Cb3(i)))/((Sref(i))*(Cbref(i))));    
% Still water power Scenario 3
Ps1(i,ij) = Psref(i,v(ij))*(((Sref(i))*(Cb1(i)))/((Sref(i))*(Cbref(i))));

% Relative vessel and wave speed        
vr(i,ij) = ((v(ij))/1.94384449)+u(i); 
            
% 3-2 Wave power (Pw)

% Reference wave power
Pwref(i,ij)=((Caw(i)*p*g*(((H13(i))/2)^2)*(B(i))^2)/(2*L(i))) * vr(i,ij);

% Adjusted wave coefficient for Scenario 3
Caw1(i) = (Caw(i)) * ((sqrt((B1(i))/(L(i)))) / (sqrt((B(i))/(L(i)))));      
% Wave power Scenario 3
Pw1(i,ij)=((Caw1(i)*p*g*(((H13(i))/2)^2)*(B1(i))^2)/(2*L(i))) * vr(i,ij);   

% 3-4 Total power (P)
% Initial total power
Pref(i,ij) = (K(i,ij))*((Psref(i,ij))+((Pwref(i,ij))))+(Paux(i));  
% Total power Scenario 1
P2(i,ij) = (K(i,ij))*((Ps2(i,ij))+((Pwref(i,ij))))+(Paux(i));  
% Total power Scenario 2
P3(i,ij) = (K(i,ij))*((Ps3(i,ij))+((Pwref(i,ij))))+(Paux(i));  
% Total power Scenario 3
P1(i,ij) = (K(i,ij))*((Ps1(i,ij))+((Pw1(i,ij))))+(Paux(i));  


% 3-5 Adjusting idle power and upper limit cut off for all scenarios
    if Pref(i,ij) < kWmin(i); 
       Pref(i,ij) = kWmin(i);
       
       P1(i,ij) < kWmin(i); 
       P1(i,ij) = kWmin(i);
       
       P2(i,ij) < kWmin(i); 
       P2(i,ij) = kWmin(i);
       
       P3(i,ij) < kWmin(i); 
       P3(i,ij) = kWmin(i);
         
    else if Pref(i,ij) > kW(i);
            Pref(i,ij) = NaN;
            
            P1(i,ij) > kW(i);
            P1(i,ij) = NaN;
            
            P2(i,ij) > kW(i);
            P2(i,ij) = NaN;
            
            P3(i,ij) > kW(i);
            P3(i,ij) = NaN;         
         end
    end
   
%% Section 4 - Fuel
% 4-1 Fuel during sailing (Fs)
% Fuel during sailing reference scenario
Fsref(i,ij) = (Kf(i))*((Pref(i,ij))*(D(i))/(v(ij)));  
% Fuel during sailing scenario 1
Fs2(i,ij) = (Kf(i))*((P2(i,ij))*(D(i))/(v(ij))); 
% Fuel during sailing scenario 2
Fs3(i,ij) = (Kf(i))*((P3(i,ij))*(D(i))/(v(ij))); 
% Fuel during sailing scenario 3
Fs1(i,ij) = (Kf(i))*((P1(i,ij))*(D(i))/(v(ij))); 

% 4-2 Fuel used in ports and slow zones (Fps)
Fps(i) = (Kf(i))*((Pps(i))*(Tps(i)));                                       

% 4-3 Total fuel (F)
% Reference scenario
Fref(i,ij) = (Fsref(i,ij)) + (Fps(i));  
% Scenario 1
F2(i,ij) = (Fs2(i,ij)) + (Fps(i));
% Scenario 2
F3(i,ij) = (Fs3(i,ij)) + (Fps(i));
% Scenario 3
F1(i,ij) = (Fs1(i,ij)) + (Fps(i));

% 4-4 Freight work

Fw(i) = (Cv(i))/((Cv(i)) + (Bv(i)));

    
%% EMISSION
%  CO2 emitted per ton nautical mile
eref(i,ij) = ( (Fref(i,ij)) / ( (Fw(i))*(D(i)) * (M(i)) ) ) * (Ke(i));                     
e1(i,ij) = ( (F1(i,ij)) / ( (Fw(i))*(D(i)) * (M(i)) ) ) * (Ke(i));     
e2(i,ij) = ( (F2(i,ij)) / ( (Fw(i))*(D(i)) * (M2(i)) ) ) * (Ke(i));     
e3(i,ij) = ( (F3(i,ij)) / ( (Fw(i))*(D(i)) * (M3(i)) ) ) * (Ke(i)); 

% Total number of voyages per year    
Voy_total(i) = ((Cv(i)) + (Bv(i)));

% Roundtrips per year
Roundtrips_year(i) = Voy_total(i) / 2;

% CO2 per roundtrip ship [tons]
eref_roundtrip(i,ij) =  eref(i,ij) * ((Fw(i)) * (D(i)) * (M(i))) / 1000000;
e1_roundtrip(i,ij) =  e1(i,ij) * ((Fw(i)) * (D(i)) * (M(i))) / 1000000;
e2_roundtrip(i,ij) =  e2(i,ij) * ((Fw(i)) * (D(i)) * (M2(i))) / 1000000;
e3_roundtrip(i,ij) =  e3(i,ij) * ((Fw(i)) * (D(i)) * (M3(i))) / 1000000;
% CO2 per year per ship [tons]
eref_year_ship(i,ij) = eref_roundtrip(i,ij) * Roundtrips_year(i);
e1_year_ship(i,ij) = e1_roundtrip(i,ij) * Roundtrips_year(i);
e2_year_ship(i,ij) = e2_roundtrip(i,ij) * Roundtrips_year(i);
e3_year_ship(i,ij) = e3_roundtrip(i,ij) * Roundtrips_year(i);
% CO2 per year per ship class [tons]
eref_year(i,ij) = eref_year_ship(i,ij) * No(i);    
e1_year(i,ij) = e1_year_ship(i,ij) * No(i);  
e2_year(i,ij) = e2_year_ship(i,ij) * No(i);  
e3_year(i,ij) = e3_year_ship(i,ij) * No(i);  


% COST
% Reference scenario
f1ref(i,ij) = 1 / ((Fw(i))*(D(i)) * ((M(i))));
f2ref(i,ij) = ((Fsref(i,ij))*((Chfo(i))/10^6))+((Fps(i))*((Cmdo(i))/10^6));
f3ref(i,ij) = (TC(i)) * ((Tt(i,ij)));
f4ref(i,ij) = (M(i)) * (CM(i)) * 0.5 * (Tt(i,ij));
f5ref(i,ij) = ( (Cir(i)) / 100 ) / 365;
f6ref(i,ij) = f4ref(i,ij) * f5ref(i,ij);
Cref(i,ij)  = f1ref(i,ij) * ((f2ref(i,ij) + f3ref(i,ij) + (f4ref(i,ij)*f5ref(i,ij))));

% Scenario 1
f12(i,ij) = 1 / ((Fw(i))*(D(i)) * ((M2(i))));
f22(i,ij) = ((Fs2(i,ij))*((Chfo(i))/10^6)) + ((Fps(i)) * ((Cmdo(i))/10^6));
f32(i,ij) = (TC2(i)) * ((Tt(i,ij)));
f42(i,ij) = (M2(i)) * (CM(i)) * 0.5 * (Tt(i,ij));
f52(i,ij) = ( (Cir(i)) / 100 ) / 365;
C2(i,ij)  = f12(i,ij) * ((f22(i,ij) + f32(i,ij) + (f42(i,ij)*f52(i,ij))));

% Scenario 2
f13(i,ij) = 1 / ((Fw(i))*(D(i)) * ((M3(i))));
f23(i,ij) = ((Fs3(i,ij))*((Chfo(i))/10^6)) + ((Fps(i)) * ((Cmdo(i))/10^6));
f33(i,ij) = (TC3(i)) * ((Tt(i,ij)));
f43(i,ij) = (M3(i)) * (CM(i)) * 0.5 * (Tt(i,ij));
f53(i,ij) = ( (Cir(i)) / 100 ) / 365;
C3(i,ij)  = f13(i,ij) * ((f23(i,ij) + f33(i,ij) + (f43(i,ij)*f53(i,ij))));

% Scenario 3
f11(i,ij) = 1 / ((Fw(i))*(D(i)) * ((M(i))));
f21(i,ij) = ((Fs1(i,ij))*((Chfo(i))/10^6)) + ((Fps(i)) * ((Cmdo(i))/10^6));
f31(i,ij) = (TC(i)) * ((Tt(i,ij)));
f41(i,ij) = (M(i)) * (CM(i)) * 0.5 * (Tt(i,ij));
f51(i,ij) = ( (Cir(i)) / 100 ) / 365;
C1(i,ij)  = f11(i,ij) * ((f21(i,ij) + f31(i,ij) + (f41(i,ij)*f51(i,ij))));
        end
end

% Extraction of emission values for maximum speeds for the different
% scenarios

for i = 1:x %1 to X number of ships
        for ij = 1:vmax %1 to vmax number of speeds
            if  isnan(eref_year(i,ij))== false
            eref_year_max(i,1)=eref_year(i,ij);
            end
        end
        
        for ij = 1:vmax %1 to vmax number of speeds
            if  isnan(e1_year(i,ij))== false
            e1_year_max(i,1)=e1_year(i,ij);
            end
        end
        
        for ij = 1:vmax %1 to vmax number of speeds
            if  isnan(e2_year(i,ij))== false
            e2_year_max(i,1)=e2_year(i,ij);
            end
        end
        
        for ij = 1:vmax %1 to vmax number of speeds
            if  isnan(e3_year(i,ij))== false
            e3_year_max(i,1)=e3_year(i,ij);
            end
        end
end



%% Results

% Dry Bulk(Panamax) - Emission
xlswrite('BlockResults.xlsx', eref(9,1:30), 'RawData', 'D3');   
xlswrite('BlockResults.xlsx', e1(9,1:30), 'RawData', 'D4');
xlswrite('BlockResults.xlsx', e2(9,1:30), 'RawData', 'D5');
xlswrite('BlockResults.xlsx', e3(9,1:30), 'RawData', 'D6');

%Crude Oil(75'-120') - Emission
xlswrite('BlockResults.xlsx', eref(27,1:30), 'RawData', 'D10');     
xlswrite('BlockResults.xlsx', e1(27,1:30), 'RawData', 'D11');
xlswrite('BlockResults.xlsx', e2(27,1:30), 'RawData', 'D12');
xlswrite('BlockResults.xlsx', e3(27,1:30), 'RawData', 'D13');

% Oil Products(15'-25') - Emission
xlswrite('BlockResults.xlsx', eref(33,1:30), 'RawData', 'D17');    
xlswrite('BlockResults.xlsx', e1(33,1:30), 'RawData', 'D18');
xlswrite('BlockResults.xlsx', e2(33,1:30), 'RawData', 'D19');
xlswrite('BlockResults.xlsx', e3(33,1:30), 'RawData', 'D20');

% Dry Bulk - Cost
xlswrite('BlockResults.xlsx', Cref(9,1:30), 'RawData', 'D24');    
xlswrite('BlockResults.xlsx', C1(9,1:30), 'RawData', 'D25');
xlswrite('BlockResults.xlsx', C2(9,1:30), 'RawData', 'D26');
xlswrite('BlockResults.xlsx', C3(9,1:30), 'RawData', 'D27');

%Crude Oil(75'-120') - Cost
xlswrite('BlockResults.xlsx', Cref(27,1:30), 'RawData', 'D31');    
xlswrite('BlockResults.xlsx', C1(27,1:30), 'RawData', 'D32');
xlswrite('BlockResults.xlsx', C2(27,1:30), 'RawData', 'D33');
xlswrite('BlockResults.xlsx', C3(27,1:30), 'RawData', 'D34');

% Oil Products(15'-25') - Cost
xlswrite('BlockResults.xlsx', Cref(33,1:30), 'RawData', 'D38');    
xlswrite('BlockResults.xlsx', C1(33,1:30), 'RawData', 'D39');
xlswrite('BlockResults.xlsx', C2(33,1:30), 'RawData', 'D40');
xlswrite('BlockResults.xlsx', C3(33,1:30), 'RawData', 'D41');


% Reference scenario maximum emission
xlswrite('BlockResults.xlsx', eref_year_max(7:13,1), 'RawData', 'D46'); 
xlswrite('BlockResults.xlsx', eref_year_max(25:30,1), 'RawData', 'D53'); 
xlswrite('BlockResults.xlsx', eref_year_max(31:36,1), 'RawData', 'D59'); 

% Maximum emission scenario 1
xlswrite('BlockResults.xlsx', e2_year_max(7:13,1), 'RawData', 'F46');
xlswrite('BlockResults.xlsx', e2_year_max(25:30,1), 'RawData', 'F53');
xlswrite('BlockResults.xlsx', e2_year_max(31:36,1), 'RawData', 'F59');

% Maximum emission scenario 2
xlswrite('BlockResults.xlsx', e3_year_max(7:13,1), 'RawData', 'G46');
xlswrite('BlockResults.xlsx', e3_year_max(25:30,1), 'RawData', 'G53');
xlswrite('BlockResults.xlsx', e3_year_max(31:36,1), 'RawData', 'G59');

% Maximum emission scenario 3
xlswrite('BlockResults.xlsx', e1_year_max(7:13,1), 'RawData', 'E46');
xlswrite('BlockResults.xlsx', e1_year_max(25:30,1), 'RawData', 'E53');
xlswrite('BlockResults.xlsx', e1_year_max(31:36,1), 'RawData', 'E59');

