%% Section 1. File read - Frontend Excel sheet with input parameters
Data =  xlsread('FrontendSpeedRed.xlsx', 2, 'D5:AK64');

% Constants and factors used with input parameters 
x    =   60;      % Number of ship type entries in Excel input document
vmax =   30;      % Number er speed iterations
LT   =   30;      % Expected life time
MCR  =   0.75;    % Maximum continous revolution at design speed
ktms =   1.852;   % Knots to m/s conversion factor
p    =   1.025;   % Density of saltwater 
g    =   9.81;    % Earths gravity
v    =   1:vmax;  % Vessel speed, 1 through 30 knots

for i = 1:x 
No(i,1)   =   Data(i,1);    % Number of ships
Voy(i,1)  =   Data(i,2);    % Distance per voyage
Day(i,1)  =   Data(i,3);    % Days per voyage
Tpsd(i,1) =   Data(i,4);    % Time in port and slow zones
vs(i,1)   =   Data(i,5);    % Vessel Speed 
vd(i,1)   =   Data(i,6);    % Design Speed
kW(i,1)   =   Data(i,7);    % Engine size (Installed power)    
n(i,1)    =   Data(i,8);    % Engine efficiency
Dis(i,1)  =   Data(i,11);   % Displacement of water (tons)
B(i,1)    =   Data(i,12);   % Width of ships at water line
L(i,1)    =   Data(i,13);   % Length of ship at water line
Dft(i,1)  =   Data(i,14);   % Draft - Fully loaded
S(i,1)    =   Data(i,15);   % Wetter surface of ship
Ct(i,1)   =   Data(i,16);   % Drag coefficient
Kf(i,1)   =   Data(i,17);   % Gram fuel per kWh
H13(i,1)  =   Data(i,18);   % Wave height
u(i,1)    =   Data(i,19);   % Wave Speed
Caw(i,1)  =   Data(i,20);   % Wave Drag
Ke(i,1)   =   Data(i,21);   % CO2 per unit fuel
M(i,1)    =   Data(i,22);   % Average weigth of cargo
Chfo(i,1) =   Data(i,23);   % Cost of heavy fuel oil
Cmdo(i,1) =   Data(i,24);   % Cost of marine diesel oil
TC(i,1)   =   Data(i,25);   % Time charter cost per day
CM(i,1)   =   Data(i,26);   % Export value per ton
Cir(i,1)  =   Data(i,27);   % Annual interest
j(i,1)    =   Data(i,28);   % Propeller constant, speed independent
k(i,1)    =   Data(i,29);   % Propeller constant, speed dependent
Cv(i,1)   =   Data(i,30);   % Cargo voyages
Bv(i,1)   =   Data(i,31);   % Ballast voyages
Lws(i,1)  =   Data(i,34);   % Lightship weight

% ADDITIONAL DATA                                             
% Distance per roundtrip;
D = 2 .* Voy;

% Time per roundtrip
T = 2 .* Day;

% Minimum / idle engine load
kWmin = 0.15 .* kW;
 
% Engine load in ports and slow zones
Pps = kWmin;

%Time in port and slow zones (days to hour conversion)
Tps(i,1) = Tpsd(i,1)*24;
end

% Calculation for time spent per voyage per boat, at different speeds
for i = 1:x %1 to X number of ships
    for h = 1:vmax %1 to vmax number of speeds
        Tt(i,h) =  ((D(i)/h)/24);     
    end   
end


%% Section 2. Propeller efficiencies and still water resistance factors
% 2-1 Change in propeller efficientcy for lower speeds
for i = 1:x %1 to X number of ships
    for ij=1:vmax
    if  v(1,ij) > vd(i,1);
        Ku(i,ij) = n(i,1);
    else
        Ku(i,ij) = n(i,1) * (j(i,1) + (k(i,1) * sqrt(v(1,ij)/vd(i,1))));
    end 

        % 2-2 Propeller efficiency
        K(i,ij) = 1 / (Ku(i,ij));   
end

% 2-3 Still Water Resistance Factor
Csref(i,1)   =   ((kW(i,1))*MCR*(1/(K(i,ij)))*2)/(p*(S(i,1))*((vd(i,1))^3)); 
end

% 2-4 Calculations for still water drag coefficients 
for i = 1:x %1 to X number of ships
    for h = 1:vmax %1 to vmax number of speeds
    if h > vd(i); 
       Cs(i,h) = Csref(i) * (0.8 + (0.2*(h/vd(i))^2));
    else
        if h < vd(i);
           Cs(i,h) = (Csref(i)*0.9) + (((h/vd(i))*0.1)*Csref(i));  
        else
           Cs(i,h) = Csref(i);
        end
    end 
    end 
end


%% Section 3. - POWER
%Iterate through 1 to X number of ships
for i = 1:x
    %Iterate through 1 to vmax number of speeds
    for ij = 1:vmax
        % 3-1 Still water power (Ps)            
        Ps(i,ij) = (p*Cs(i,ij)*S(i)*((v(ij))^3))/2;

        %Relative vessel and wave speed (used in wave power (Pw))
        Vr(i,ij) = ((v(ij))/1.94384449)+u(i);

        % 3-2 Wave power (Pw)     
        Pw(i,ij) = ((Caw(i)*p*g*(((H13(i))/2)^2)*(B(i))^2)/(2*L(i))) * Vr(i,ij);   

        % 3-3 Auxiliary power (Paux) 
    if kW(i) > 10000;
       Paux(i,1) = 250 + (kW(i) * 0.025);
    else
       Paux(i,1) = kW(i) * 0.05;
    end

    % 3-4 Total power (P)
    P(i,ij) = (K(i,ij))*((Ps(i,ij))+((Pw(i,ij))))+(Paux(i));  

    % 3-5 Adjusting idle power and upper limit cut off
    if P(i,ij) < kWmin(i); 
       P(i,ij) = kWmin(i);
         
    else if P(i,ij) > kW(i);
            P(i,ij) = NaN; 
         end
    end
    end
end

%% Section 4. - FUEL
%Iterate through 1 to X number of ships
for i = 1:x
        %Iterate through 1 to vmax number of speeds
    for ij = 1:vmax
    % 4-1 Fuel during sailing (Fs)
    Fs(i,ij) = (Kf(i))*((P(i,ij))*(D(i))/(v(ij)));

    % 4-2 Fuel used in ports and slow zones (Fps)
    Fps(i) = (Kf(i))*((Pps(i))*(Tps(i)));

    % 4-3 Total fuel (F)
    F(i,ij) = (Fs(i,ij)) + (Fps(i));

    % 4-4 Freight work factor
        % Total number of voyages per year    
        Voy_total(i) = ((Cv(i)) + (Bv(i)));

        % Roundtrips per year
        Roundtrips_year(i) = Voy_total(i) / 2;
    
        % Freight work factor (Fw)
        Fw(i,1) = (Cv(i))/(Voy_total(i));
    end
end


%% Section 5. - LCA emissions and cost

%Section 5.1 - Life Cycle Assessment of ships (All values in tonnes)
Rps = 0.100;   %Replacement steel factor needed for repairs
Rmf = 2.660;   %Raw material factor
Atdr = 3484;   %Average transport distance of raw materials (Nautical miles)
Atds = 4136;   %Average transport distance of scrap materials (Nautical miles)
Sfc = 1.750;   %CO2 Steel fabrication coefficient
Sbc = 0.216;   %CO2 Ship building coefficient
Rpc = 0.303;   %CO2 Repair coefficient
Rec = 0.180;   %CO2 Scrapping coefficient
Tnc = 12.00;   %CO2 gram per ton nm coefficient (from Panamax) 
Fus = 0.250;   %CO2 gram emitted upstream per gram fuel used

%Iterate through 1 to X number of ships
for i = 1:x
    %Iterate through 1 to vmax number of speeds
    for ij = 1:vmax

%5-1-2 Amount of steel needed for repairs (Lwsr)
Lwsr(i,1) = (Lws(i,1)) * Rps;
%5-1-3 Amount of steel in ship + steel need for repair over life time(Lwst)
Lwst(i,1) = (Lws(i,1)) + (Lwsr(i,1));
%5-1-4 Raw materials need for extraction and transport (Rawm)
Rawm(i,1) = Lwst(i,1) * Rmf;
%5-1-5 Tonnes nautical miles (Raw materials) (Tnmr)
Tnmr(i,1) = Rawm(i,1) * Atdr;
%5-1-6 Tonnes nautical miles (Scrap materials) (Tnms)
Tnms(i,1) = Lwst(i,1) * Atds;               
    
%5-1-8 Ship Building
%CO2 emissions from ship building per ship
CO2sb(i,1) = Lwst(i,1) * Sbc;
%Sum of CO2 for number of ships
CO2sb_class(i,1) = CO2sb(i,1) * No(i);
%CO2 emission per year per class
CO2sb_year(i,1) = CO2sb_class(i,1) / LT;

%5-1-9 Steel fabrication
%CO2 emission from steel fabrication per ship
CO2sf(i,1) = Lwst(i,1) * Sfc;
%Sum of CO2 for number of ships
CO2sf_class(i,1) = CO2sf(i,1) * No(i);
%CO2 emission per year per class
CO2sf_year(i,1) = CO2sf_class(i,1) / LT;
    
%5-1-10 Repairs
%CO2 emissions from repair per ship
CO2rp(i,1) = Lwsr(i,1) * Rpc;
%Sum of CO2 for number of ships
CO2rp_class(i,1) = CO2rp(i,1) * No(i);
%CO2 emission per year per class
CO2rp_year(i,1) = CO2rp_class(i,1) / LT;
    
%5-1-11 Scrapping
%CO2 emissions from cutting of steel scrapping per ship
CO2re(i,1) = Lws(i,1) * Rec;
%Sum of CO2 for number of ships
CO2re_class(i,1) = CO2re(i,1) * No(i);
%CO2 emission per year per class
CO2re_year(i,1) = CO2re_class(i,1) / LT;
    
% Construction related Transport
%CO2 emissions from transportation of raw materials per ship
CO2tra(i,1) = (Tnmr(i,1) * Tnc) / 1000000;
%Sum of CO2 for number of ships
CO2tra_class(i,1) = CO2tra(i,1) * No(i);
%CO2 emission per year per class
CO2tra_year(i,1) = CO2tra_class(i,1) / LT;
    
% End of life related transport
%CO2 emissions from transportation of scrap materials
CO2tsc(i,1) = (Tnms(i,1) * Tnc) / 1000000;
%Sum of CO2 for number of ships
CO2tsc_class(i,1) = CO2tsc(i,1) * No(i);
%CO2 emission per year per class
CO2tsc_year(i,1) = CO2tsc_class(i,1) / LT;
    
% Totals 
%5-1-7 Total transport 
CO2tr(i,1) = CO2tra(i,1) + CO2tsc(i,1); %Total CO2 emission from transportation
%5-1-13 Total CO2 in tonnes
CO2LCA_tonnes(i,1) = CO2sf(i,1) + CO2sb(i,1) + CO2rp(i,1) + CO2re(i,1) + CO2tr(i,1);
%Total CO2 in grams
CO2LCA(i,1) = CO2LCA_tonnes(i,1) * 1000000;

% EMISSION INDIVIDUAL PARTS
%5-1-12 CO2 from Upstream fuel production per ship per round trip
e_upstream_roundtrip(i,ij) = F(i,ij) * Fus;
% CO2 from upstream fuel production per year per class
e_upstream_year_class(i,ij) = e_upstream_roundtrip(i,ij) * Roundtrips_year(i) * No(i);

% CO2 from shipbuilding and EOL per year
CO2LCA_year(i,1) = CO2LCA(i) / LT;
% CO2 from shipbuilding and OEL per roundtrip
CO2LCA_roundtrip(i,1) = CO2LCA_year(i) / Roundtrips_year(i);

% CO2 from combustion per year per ship
e_combustion_year(i,ij) = F(i,ij) * Ke(i) * Roundtrips_year(i) ;
% CO2 from conbustion per year per class
e_combustion_year_class(i,ij) = e_combustion_year(i,ij) * No(i);

% CO2 from combustion normalized to freightwork
e_combustion(i,ij) = (F(i,ij) / (Fw(i)*D(i) * M(i)))  * (Ke(i));
% CO2 from upstream fuel production normalized to freightwork
e_upstream(i,ij) = e_upstream_roundtrip(i,ij) / ( (Fw(i))*(D(i)) * (M(i)) );
% CO2 from LCA normalized to freightwork
e_LCA(i,1) = CO2LCA_roundtrip(i) / ((Fw(i)) * (D(i)) * (M(i))); 

%Section 5.2 Emission all parts combined
% CO2 emitted per ton nautical mile [gram / ton nm]
e(i,ij) = e_combustion(i,ij) + e_upstream(i,ij) + e_LCA(i);
% CO2 per roundtrip ship [tons]
e_roundtrip(i,ij) =  e(i,ij) * ((Fw(i)) * (D(i)) * (M(i))) / 1000000;
% CO2 per year per ship [tons]
e_year_ship(i,ij) = e_roundtrip(i,ij) * Roundtrips_year(i);
% CO2 per year per ship class [tons]
e_year(i,ij) = e_year_ship(i,ij) * No(i);
% CO2 per life time per ship [tons]
e_lifetime(i,ij) = e_year_ship(i,ij) * LT;
% CO2 per life time per ship class [tons]
e_class(i,ij) = e_lifetime(i,ij) * No(i);

% Section 5.3 Cost
% Normalization factor for ton nautical mile
f1(i,ij) = 1 / ((Fw(i))*(D(i)) * ((M(i))));
% Fuel cost of fuel consumption during sailing & ports and slow zones
f2(i,ij) = ((Fs(i,ij)) * ((Chfo(i))/10^6)) + ((Fps(i)) * ((Cmdo(i))/10^6));
% Time charter cost
f3(i,ij) = (TC(i)) * ((Tt(i,ij)));
% Value of cargo transported
f4(i,ij) = (M(i)) * (CM(i)) * 0.5 * (Tt(i,ij));
% Interest rate on captial invested
f5(i,ij) = ( (Cir(i)) / 100 ) / 365;
% Capital investment cost
f6(i,ij) = f4(i,ij) * f5(i,ij);                                                                           
%Cost per ship per ton nautical mile
C(i,ij)  = f1(i,ij) * (f2(i,ij) + f3(i,ij) + (f4(i,ij)*f5(i,ij)));
% Cost per ship per round trip
C_roundtrip(i,ij)  = ((f2(i,ij) + f3(i,ij) + (f4(i,ij)*f5(i,ij))));
% Cost per ship per year
C_year(i,ij) = C_roundtrip(i,ij) * Roundtrips_year(i);
% Cost per class per year
C_year_class(i,ij) = C_year(i,ij) * No(i);

        end
end

%%Section 6. - Results 
%1 to X number of ships
for i = 1:x
% Emission per ton nm at design speed only
e_vd(i,1) = e(i,vd(i));
end

for i = 1:x %1 to X number of ships
%Lifetime emission at design speed only
e_lifetime_vd(i,1) = e_lifetime(i,vd(i)); 
e_class_vd(i,1) = e_class(i,vd(i));
% e_upstream_year_max(i,1) = max
end

% Emission reduction scenarios at Vd, Vd-1, Vd-2
for i = 1:x %1 to X number of ships
        for ij = 1:vmax %1 to vmax number of speeds
            if  isnan(e_year(i,ij))== false
            e_year_max(i,1)=e_year(i,ij);
            end
        end
        
        for ij = 1:25
            if  isnan(e_year(i,ij+1))== false
            e_year_middle(i,1)=e_year(i,ij);
            end
        end
        
        for ij = 1:25
             if  isnan(e_year(i,ij+2))== false
            e_year_best(i,1)=e_year(i,ij);
             end
        end
        
        for ij = 1:vmax %1 to vmax number of speeds
            if  isnan(e_combustion_year_class(i,ij))== false
            e_combustion_year_class_max(i,1)=e_combustion_year_class(i,ij)/ 1000000;
            end
        end
        
         for ij = 1:25
            if  isnan(e_combustion_year_class(i,ij+1))== false
            e_combustion_year_class_middle(i,1)=e_combustion_year_class(i,ij)/ 1000000;
            end
        end
        
        for ij = 1:25
             if  isnan(e_combustion_year_class(i,ij+2))== false
            e_combustion_year_class_best(i,1)=e_combustion_year_class(i,ij)/ 1000000;
             end
        end
        
        for ij = 1:vmax %1 to vmax number of speeds
            if  isnan(e_upstream_year_class(i,ij))== false
            e_upstream_year_class_max(i,1)=e_upstream_year_class(i,ij)/ 1000000;
            end
        end
        
        for ij = 1:25
            if  isnan(e_upstream_year_class(i,ij+1))== false
            e_upstream_year_class_middle(i,1)=e_upstream_year_class(i,ij)/ 1000000;
            end
        end
        
        for ij = 1:25
             if  isnan(e_upstream_year_class(i,ij+2))== false
            e_upstream_year_class_best(i,1)=e_upstream_year_class(i,ij)/ 1000000;
             end
        end
   
end

%Annual freightwork 
for i = 1:x
    Freighwork_year(i,1)  = Fw(i) * D(i) * M(i) * Roundtrips_year(i) * No(i);
end

